﻿; ActivePseudo.ahk[UTF-8] (2024/06/12 11:00)
; for AutoHotKey v2.0.12.0 or later
;
; 仮想デスクトップ上に起動されたVivaldiを一定周期ごとにアクティブに切り替えて
; 低プライオリティ動作でタイマイベント類が滞留することで起きるタブのクラッシュ
; を抑止します。
;
; 使い方
;  ActivePseudo.exe
;
; マウスが動かない時間が一定時間以上になったら１つめのフルパスをインターバル
; 秒後にアクティブ化し、それを繰り返し回数ぶん繰り返したら２つめのフルパスに
; 移り最後のパスまで処理されると１つめのパスに戻り全体の処理を繰り返します。
; 途中でマウスが動くと中断し、また一定時間操作されないと動作を再開します。
;
; ※コマンドラインオプションからiniファイルに変更しました


/* 非表示になっているアプリも対象とする（仮想デスクトップ上では非表示扱い？） */
#WinActivateForce
DetectHiddenWindows True

;//ToolTipはスクリーンを対象とする
CoordMode("ToolTip", "Screen")

;//MouseGetPosはスクリーンを対象とする
CoordMode("Mouse", "Screen")


/* タスクトレイアイコンにメニュー項目追加 */
A_TrayMenu.Insert("1&", "ExecForce after 5sec", handleMenuItem)



/* 変数初期化 */
IniFile       := "ActivePseudo.ini"
IniSect       := "APP"
POS_LIMIT     := 5
lastTime      := A_Now
lastX         := -1
lastY         := -1
idx           := 1
page          := 1
modeActive    := 1 ;// 1=マウスが動いてる, 0=一定時間以上止まっている
pathArray     := Array()
tabsArray     := Array()
repsArray     := Array()
keysArray     := Array()

noDialog      := Number(IniRead(IniFile, IniSect, "NoDialog", "0"))
noTgtDialog   := Number(IniRead(IniFile, IniSect, "NoTgtDialog", "0"))
maxPath       := Number(IniRead(IniFile, IniSect, "MAX", "1"))
intervalSec   := Number(IniRead(IniFile, IniSect, "Interval", "10"))
pastMin       := Number(IniRead(IniFile, IniSect, "PastMin", "10"))
msgTime       := Number(IniRead(IniFile, IniSect, "MsgTime", "1"))
tipsOffset    := Number(IniRead(IniFile, IniSect, "TipsOffset", "140"))
tipsDisplayS  := Number(IniRead(IniFile, IniSect, "TipsDisplayS", "-1600"))
tipsDisplayL  := Number(IniRead(IniFile, IniSect, "TipsDisplayL", "1920"))
displayLimitS := Number(IniRead(IniFile, IniSect, "DisplayLimitS", "0"))
displayLimitL := Number(IniRead(IniFile, IniSect, "DisplayLimitL", "1920"))
tipsDisplayOffset := 0
settingMin  := pastMin

while(maxPath >= idx){
	strTemp := IniRead(IniFile, IniSect, "PATH" String(idx), "")
	strTemp := StrReplace(strTemp, '"', "")
	pathArray.Push(strTemp)

	;繰り返し回数の読み込み(REPS1～)
	strTemp := IniRead(IniFile, IniSect, "REPS" String(idx), "")
	strTemp := StrReplace(strTemp, '"', "")
	repsArray.Push(strTemp)

	;キー送出設定の読み込み(KEYS1～)
	strTemp := IniRead(IniFile, IniSect, "KEYS" String(idx), "")
	strTemp := StrReplace(strTemp, '"', "")
	keysArray.Push(strTemp)

	;タブ切り替え定義の読み込み(TABS1～)
	strTemp := IniRead(IniFile, IniSect, "TABS" String(idx), "")
	;両端のダブルクォートを削除し、カンマ区切りを配列に分割
	strTemp := StrReplace(strTemp, '"', "")
	aryTemp := StrSplit(strTemp, ",")
	if(aryTemp.Length < repsArray[idx]){
		aryTemp.Length := repsArray[idx]
	}
	tabsArray.Push(aryTemp)

	;MsgBox("Path" idx " = " pathArray[idx])
	idx++
}

;//GetCursorPos用のバッファを獲得
point       := Buffer(8)

if(not noDialog){
	MsgBox("ウィンドウ切り替えの常駐を開始します",,"0x40040 T3")
}

idx := 1
targetPath := pathArray[idx]
repeatTimes := repsArray[idx]
sendKeys := keysArray[idx]
notFound := 0

/* 順次アクティブウィンドウを切り替え */
Loop
{
	count := repeatTimes
	while( count > 0 ){
		chkMove()
		notFound := 0
		if( modeActive = 0 ){
			if( StrLen(targetPath) > 0 ){
				try {
					if( InStr( targetPath, "fox") ){
						pathStr := "ahk_exe " targetPath " ahk_class MozillaWindowClass"
					}else{
						pathStr := "ahk_exe " targetPath " ahk_class Chrome_WidgetWin_1"
					}
					WinActivate( pathStr )
				} catch TargetError as err {
					if( noTgtDialog = 0 ){
						MsgBox( targetPath " not found", , "0x40010 T3")
					}
					notFound := 1
				}
				hWnd := WinExist( pathStr )
				if( hWnd > 0 ){
					try{
						WinGetPos( &OutX, &OutY, &OutW, &OutH, "ahk_id " hWnd )
					} catch Error as err {
						hWnd := 0
						OutX := 0
						OutY := 0
						OutW := 0
						OutH := 0
					}
					if( OutX < displayLimitS ){
						TipsDisplayOffset := tipsDisplayS
					}else if( OutX > displayLimitL ){
						TipsDisplayOffset := tipsDisplayL
					}else{
						TipsDisplayOffset := 0
					}
				}
			}else{
				MsgBox( "PATH" idx " setting error", , "0x40010 T3")
			}
			tim := repeatTimes - count + 1
			count := count - 1
			if(sendKeys = 1){
				try{
					if( tabsArray[idx][tim] != "0" ){
						Sleep(1000)
						Send("^" tabsArray[idx][tim])
					}
				} catch Error as err {
					MsgBox( "TABS" idx "[" tim "] setting error", , "0x40010 T3")
				}
			}
			if(sendKeys = 2){
				try{
					Sleep(1000)
					Send("{F5}")
				} catch Error as err {
					MsgBox( "F5 setting error", , "0x40010 T3")
				}
			}
		}
		if( notFound = 0 ){
			sleepLoop()
		}
	}
	if( modeActive = 0 ){
		idx := idx + 1
		if( maxPath < idx ){
			idx := 1
		}
		page := idx
		targetPath := pathArray[idx]
		repeatTimes := repsArray[idx]
		sendKeys := keysArray[idx]
	}
}

/* 座標を獲得して前回値と比較する */
chkMove(){
	global point,lastTime,lastX,lastY,modeActive,pastMin
	try {
		MouseGetPos( &posX, &posY, &posWin, &posCtrl )
	} catch Error as err {
		return
	}
/*
	DllCall("GetCursorPos", "Ptr", point, "Int")
	posX := NumGet(point, 0, "Int")
	posY := NumGet(point, 4, "Int")
*/
	if( Abs(posX - lastX) > POS_LIMIT or Abs(posY - lastY) > POS_LIMIT ){
		lastX := posX
		lastY := posY
		lastTime := A_Now
		if( modeActive = 0 ){
			ToolTip( ,,, 1 )
			if(msgTime > 0){
				MsgBox( "自動切り替えを中断します", , "0x40030 T" msgTime)
			}
			pastMin := settingMin
		}
		modeActive := 1
	}
	past := DateDiff( A_Now, lastTime, "Seconds" )
	if( past > pastMin * 60 ){
		if( modeActive = 1 ){
			ToolTip( ,,, 1 )
			if(msgTime > 0){
				MsgBox( "自動切り替えを開始します", , "0x40040 T" msgTime)
			}
		}
		modeActive := 0
	}
	;MsgBox( "past=" past " POS X=" posX ",Y=" posY, , "0x40000 T1" )
	
	if( modeActive = 0 and repeatTimes != count ){
		ToolTip( "[ActivePseudo] page=" page " count=" repeatTimes - count "/" repeatTimes "`n"
				. "path=" targetPath, A_ScreenWidth / 2 - tipsOffset + tipsDisplayOffset, 0, 1 )
	}
}

/* 途中で座標を確認しながら待機する */
sleepLoop(){
	global modeActive,count,intervalSec
	sleepCount := intervalSec * 1000
	while( sleepCount > 0 ){
		lastActive := modeActive
		chkMove()
		if( modeActive = 1 and lastActive = 0 ){
			;count := 0
			sleepCount := 0
		}else{
			sleepCount := sleepCount - 100
			Sleep(100)
		}
	}
}

/* メニューからの呼び出し */
handleMenuItem(ItemName, ItemPos, MyMenu){
	global pastMin,settingMin
	if( pastMin != 0.001 ){
		MsgBox( "5秒後に自動切り替えを開始します", , "0x40040 T5")
		pastMin := 0.001
	}
}




